
#include "Framework/Defs.h"

#include "Framework/Gfx.h"
#include "Framework/TextureManager.h"
#include "Framework/BlitManager.h"
#include "Framework/Keyboard.h"
#include "Framework/Timers.h"
#include "Framework/Sound.h"
#include "Framework/Utils.h"

#include "Game/GameScreen.h"
#include "Game/ImageScreen.h"
#include "Game/MenuScreen.h"

bool init_gfx(HINSTANCE hInst)
{
	new Gfx;

	GFX->set_window_mode(false);
	if (!GFX->init(hInst)) return false;
	
	new TextureManager(GFX->get_device());
	new BlitManager(GFX->get_device());

	return true;
}

bool init_input(HINSTANCE hInst)
{
	new Keyboard;
	
	if (!KEY->init(hInst)) return false;

	return true;
}

bool init_sound()
{
	new Sound;

	if (!SOUND->init()) 
	{
		ERROR_MSG("Couldn't start sound... Swarm will be silent...");
	}

	return true;
}

bool init(HINSTANCE hInst)
{
	if (!init_gfx(hInst)) return false;
	if (!init_input(hInst)) return false;
	if (!init_sound()) return false;

	new Timers;

	return true;
}

void shutdown()
{
	delete SOUND;
	delete TIMERS;
	delete KEY;
	delete BLIT;
	delete TEXTURES;
	delete GFX;
}

INT WINAPI WinMain(HINSTANCE hInst,HINSTANCE,LPSTR,INT)
{
	if (!init(hInst)) return -1;
	
	ImageScreen* img=new ImageScreen;
	if (!img->init("images/credits.jpg")) return -1;
	img->load_assets("data/SoundAssets.txt");
	SOUND->play(SOUND->find_by_name("IntroMusic"));
	img->run();
	delete img;

	img=new ImageScreen;
	if (!img->init("images/title.jpg")) return -1;
	img->run();
	delete img;

	bool exit=false;
	while (!exit)
	{
		MenuScreen* menu=new MenuScreen;
		if (!menu->init()) return -1;
		long opt=menu->run();
		delete menu;

		if (opt==0)
		{
			bool exit=false;
			long level=1;
			while (!exit)
			{
				GameScreen *scr=new GameScreen;
				if (!scr->init(level)) return -1;
				opt=scr->run();
				scr->shutdown();
				delete scr;
				if (opt==0)
				{
					exit=true;
				}
				else if (opt==1)
				{
					img=new ImageScreen;
					if (!img->init("images/gameover.jpg")) return -1;
					img->run();
					delete img;
					exit=true;
				}
				else if (opt==2)
				{
					img=new ImageScreen;
					if (level<12)
					{
						if (!img->init("images/next_level.jpg")) return -1;
					}
					else
					{
						if (!img->init("images/endgame.jpg")) return -1;
						exit=true;
					}
					img->run();
					delete img;
					level++;
				}
			};
		}
		else if (opt==1)
		{
			img=new ImageScreen;
			if (!img->init("images/inst01.jpg")) return -1;
			img->run();
			delete img;
			img=new ImageScreen;
			if (!img->init("images/inst02.jpg")) return -1;
			img->run();
			delete img;
		}
		else if (opt==2)
		{
			exit=true;
		}
	}

	shutdown();

    return 0;
}
