// Gfx.h: interface for the Gfx class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_GFX_H__65AF6A21_DD51_4626_9C30_4141D1E621D0__INCLUDED_)
#define AFX_GFX_H__65AF6A21_DD51_4626_9C30_4141D1E621D0__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <d3d9.h>
#include <d3dx9.h>
#include "dx/dxutil.h"
#include "dx/d3denumeration.h"
#include "dx/d3dsettings.h"

class Gfx  
{
public:
	Gfx();
	virtual ~Gfx();

	// Init stuff and config stuff
			bool				init(HINSTANCE instance);
			void				shutdown();
	inline	void				set_window_mode(bool b) { _window_mode=b; }
	inline	bool				is_windowed() { return _window_mode; }
	inline	void				set_resolution(long rx,long ry) { _resolution=D3DXVECTOR2(float(rx),float(ry)); }
	inline	D3DXVECTOR2			get_resolution() { return _resolution; }
	inline	void				set_window(HWND window) { _window=window; }
	inline	HWND				get_window() { return _window; }
	inline	LPDIRECT3DDEVICE9	get_device() { return _device; }
	inline	unsigned long		get_window_style() { return _window_style; }

	// Real gfx stuff
			void		set_2d_render();
			void		set_camera(float x,float y,float z);
			void		clear(DWORD clear_buffers,float r=0.0f,float g=0.0f,float b=0.0f,float a=1.0f,float zVal=1.0f,unsigned long sVal=0);
			void		begin_scene();
			void		end_scene();
			void		swap_buffers();
	inline	void		set_color_modulation(float r,float g,float b) { _modulate.x=r; _modulate.y=g; _modulate.z=b; }
			D3DXVECTOR3	get_color_modulation() { return _modulate; }

public:
			void		set_cursor_position(long x,long y);

protected:
    static	bool		confirm_device_helper(D3DCAPS9* pCaps,VertexProcessingType vertexProcessingType,D3DFORMAT backBufferFormat,D3DFORMAT format);
			HRESULT		choose_initial_settings();
			HRESULT		init_d3d_environment();
			HRESULT		confirm_device(D3DCAPS9* pCaps,DWORD dwBehavior,D3DFORMAT Format);
			bool		find_best_fullscreen_mode(bool bRequireHAL,bool bRequireREF);
			bool		find_best_windowed_mode(bool bRequireHAL,bool bRequireREF);
			void		build_present_params_from_settings();

protected:
	bool				_window_mode;
	D3DXVECTOR2			_resolution;
	D3DXVECTOR3			_modulate;
protected:
    LPDIRECT3D9				_d3d9;          // The main D3D object
    LPDIRECT3DDEVICE9		_device;        // The D3D rendering device
    CD3DEnumeration			_d3d_enumeration;
	HWND					_window;	
	CD3DSettings			_d3d_settings;
	D3DPRESENT_PARAMETERS	_d3dpp;			   // Parameters for CreateDevice/Reset
    DWORD					_window_style;     // Saved window style for mode switches
    RECT					_window_bounds;    // Saved window bounds for mode switches
    RECT					_window_client;    // Saved client area size for mode switches
    DWORD					_create_flags;     // Indicate sw or hw vertex processing
    D3DCAPS9				_caps;			   // Caps for the device
    D3DSURFACE_DESC			_back_buffer_desc;   // Surface desc of the backbuffer
};

extern Gfx*		GFX;

#endif // !defined(AFX_GFX_H__65AF6A21_DD51_4626_9C30_4141D1E621D0__INCLUDED_)
