// Sprite.cpp: implementation of the Sprite class.
//
//////////////////////////////////////////////////////////////////////

#include "Sprite.h"
#include "BlitManager.h"
#include "TextureManager.h"
#include "Gfx.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Sprite::Sprite()
{
	_scale=1.0f;
	_angle=0.0f;
	_animation_last_frame_time=timeGetTime();
	set_hotspot(0.0f,0.0f);
	set_color(1.0f,1.0f,1.0f,1.0f);
}

Sprite::Sprite(float x,float y,float z)
{
	_pos.x=x;
	_pos.y=y;
	_pos.z=z;
	_scale=1.0f;
	_angle=0.0f;
	_animation_last_frame_time=timeGetTime();
	set_hotspot(0.0f,0.0f);
	set_color(1.0f,1.0f,1.0f,1.0f);
}

Sprite::~Sprite()
{
	_animation_frames.clear();
}

void Sprite::anim()
{
	if (timeGetTime()-_animation_last_frame_time>_animation_rate)
	{
		if (_animation_frames.size()>0)
		{
			_current_image++;
			if (_current_image>=_animation_frames.size())
			{
				if (_animation_loop) _current_image=0;
				else _current_image--;
			}
		}
		_animation_last_frame_time=timeGetTime();
	}

	_last_frame_time=timeGetTime();
}

void Sprite::draw()
{
	if (!_draw) return;
	if (_scale==0.0f) return;

	long tex;
	if (_animation_frames.size()>0) tex=_animation_frames[_current_image];
	else tex=_current_image;
	float sx=TEXTURES->get_size_x(tex)*_scale;
	float sy=TEXTURES->get_size_y(tex)*_scale;
	D3DXVECTOR3 color=GFX->get_color_modulation();
	color.x*=_color.x;
	color.y*=_color.y;
	color.z*=_color.z;
	if (_angle==0.0f)
	{
		BLIT->add(tex,_pos.x-_hotspot.x*_scale,_pos.y-_hotspot.y*_scale,_pos.z,sx,sy,color.x,color.y,color.z,_color.w);
	}
	else
	{
		BLIT->add(tex,_pos,_hotspot,_angle,sx,sy,color.x,color.y,color.z,_color.w);
	}
}
