// TextureManager.cpp: implementation of the TextureManager class.
//
//////////////////////////////////////////////////////////////////////

#include "TextureManager.h"
#include "Utils.h"
#include <d3dx9.h>

TextureManager* TEXTURES=NULL;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TextureManager::TextureManager(LPDIRECT3DDEVICE9 device)
{
	_device=device;
	if (!TEXTURES) TEXTURES=this;
}

TextureManager::~TextureManager()
{
	clear_all();
	if (TEXTURES==this) TEXTURES=NULL;
}

long TextureManager::load(char *name,char *filename)
{
	LPDIRECT3DTEXTURE9	texture;

	FILE *file=fopen(filename,"rb");
	if (!file)
	{
		char buffer[512];
		char working_dir[512];
		GetCurrentDirectory(512,(char*)&working_dir);
		sprintf((char*)&buffer,"File not found (%s/%s)!",(char*)&working_dir,filename);
		ERROR_MSG((char*)&buffer);
		return -1;
	}
	fclose(file);
	HRESULT hRet=D3DXCreateTextureFromFile(_device,filename,&texture);
	if (hRet!=D3D_OK)
	{
		ERROR_MSG("Can't read texture!");
		return -1;
	}

	Texture* tex=new Texture;
	tex->filename=strdup(filename);
	if (name) tex->name=strdup(name); else tex->name=NULL;
	tex->texture=texture;

	D3DSURFACE_DESC	desc;
	texture->GetLevelDesc(0,&desc);

	tex->sx=desc.Width;
	tex->sy=desc.Height;

	_textures.push_back(tex);

	return _textures.size()-1;
}

long TextureManager::add(char *name,LPDIRECT3DTEXTURE9 texture)
{
	Texture* tex=new Texture;
	tex->filename=NULL;
	if (name) tex->name=strdup(name); else tex->name=NULL;
	tex->texture=texture;

	D3DSURFACE_DESC	desc;
	texture->GetLevelDesc(0,&desc);

	tex->sx=desc.Width;
	tex->sy=desc.Height;

	_textures.push_back(tex);

	return _textures.size()-1;
}

long TextureManager::find_by_name(char *name)
{
	for (long i=0; i<_textures.size(); i++)
	{
		if (_textures[i]) 
		{
			if (_textures[i]->name)
			{
				if (!stricmp(_textures[i]->name,name)) return i;
			}
		}
	}

	return -1;
}

void TextureManager::clear_all()
{
	for (long i=0; i<_textures.size(); i++)
	{
		if (_textures[i]) 
		{
			free(_textures[i]->filename);
			_textures[i]->texture->Release();
			delete _textures[i];
		}
	}
	_textures.clear();
}

LPDIRECT3DTEXTURE9 TextureManager::get_texture(long id)
{
	if (id==-1) return NULL;
	if (_textures[id])
	{
		return _textures[id]->texture;
	}

	return NULL;
}

int	TextureManager::get_size_x(long id)
{
	if (id==-1) return 0;
	if (_textures[id]) return _textures[id]->sx;

	return 0;
}

int	TextureManager::get_size_y(long id)
{
	if (id==-1) return 0;
	if (_textures[id]) return _textures[id]->sy;

	return 0;
}
