#include "../framework/TextureManager.h"
#include "../framework/Utils.h"
#include "../framework/ParticleSystem.h"
#include "../framework/Sound.h"

#include "Ant.h"
#include "AntHill.h"
#include "Leaf.h"
#include "Grasshoper.h"
#include "GameScreen.h"
#include "PhenoromeMap.h"

Ant::Ant(float x,float y,float z) : Sprite(x,y,z)
{
	add_animation_frame(TEXTURES->find_by_name("Ant01"));
	add_animation_frame(TEXTURES->find_by_name("Ant02"));
	add_animation_frame(TEXTURES->find_by_name("Ant03"));
	add_animation_frame(TEXTURES->find_by_name("Ant02"));
	set_animation_loop(true);
	set_animation_rate(200);
	set_angle(get_random_number(0,360));
	set_hotspot(16,16);
	_goto_pos.x=x;
	_goto_pos.y=y;

	wander();
}

Ant::~Ant(void)
{
	_goto_queue.clear();
}

void Ant::anim()
{
	Sprite::anim();

	GameScreen* scr=dynamic_cast<GameScreen*>(SCREEN);
	switch (_state)
	{
		case ANT_STOP:
			{
			}
			break;
		case ANT_FOLLOW:
		case ANT_WANDER:
			{
				// Check if there is a smell around this position
				if (scr)
				{
					PhenoromeMap *pm=scr->get_phenorome_map();
					D3DXVECTOR2	p;
					bool smell=pm->get_scent(_pos.x,_pos.y,&p,this);
					if (smell)
					{
						_state=ANT_FOLLOW;
						_goto_pos.x=p.x;
						_goto_pos.y=p.y;
						_speed=1.5f;
					}
					else
					{
						if (_state==ANT_FOLLOW) 
						{
							float d;
							Leaf* leaf=scr->find_nearest_leaf(_pos.x,_pos.y,&d);
							if (d<Sqr(80.0f))
							{
								go_to(leaf->get_x_pos(),leaf->get_y_pos(),1.5f);
							}
							else wander();
						}
					}
				}
			}
			break;
		case ANT_GOTO:
			{
			}
			break;

	}

	// Check if it's already at the target pos
	float d=Sqr(_goto_pos.x-_pos.x)+Sqr(_goto_pos.y-_pos.y);
	if (d<Sqr(2))
	{
		if (_goto_queue.empty())
		{
			// Reached the position
			if (_state==ANT_WANDER)
			{
				wander();
			}
			else if (_state==ANT_GOTO)
			{
				if (scr)
				{
					if (scr->get_selected_ant()!=this) wander();
				}
			}
			else if (_state==ANT_RETURN)
			{
				wander();
			}
			else
			{
				_speed=0.0f;
			}
		}
		else
		{
			_goto_pos=_goto_queue.front();
			_goto_queue.pop_front();
		}
	}
	else
	{
		D3DXVECTOR3 old_pos=_pos;
		D3DXVECTOR2	vec;
		vec.x=_goto_pos.x-_pos.x;
		vec.y=_goto_pos.y-_pos.y;
		D3DXVECTOR2	norm_vec;
		D3DXVec2Normalize(&norm_vec,&vec);
		_pos.x=_pos.x+norm_vec.x*_speed;
		_pos.y=_pos.y+norm_vec.y*_speed;

		// Check if we come too close to an anthill and slide in that case
		if (_state==ANT_RETURN)
		{
			AntHill* ant_hill=scr->find_nearest_anthill(_pos.x,_pos.y);
			if (ant_hill)
			{
				float d=Sqr(ant_hill->get_x_pos()-_pos.x)+Sqr(ant_hill->get_y_pos()-_pos.y);
				if (d<Sqr(60.0f))
				{
					// Drop food
					ant_hill->drop_food();
					SOUND->play(SOUND->find_by_name("DropLeaf"));
					// Particle system ftw
					ParticleSystem *part=new ParticleSystem;
					part->set_pos(_pos.x,_pos.y,_pos.z-0.1f);
					part->set_image(TEXTURES->find_by_name("ParticleRound"));
					part->set_life(250.0f);
					part->set_rate(200.0f,200.0f);
					part->set_particle_life(750.0f,250.0f);
					part->set_particle_speed(0.00005f,0.000025f);
					part->set_particle_size(0.005f,0.01f);
					part->set_particle_color_start(1.0f,0.0f,0.0f,1.0f);
					part->set_particle_color_end(1.0f,0.0f,0.0f,0.2f);
					part->set_blend_mode(BLEND_ALPHA);
					SCREEN->add_drawable(part);
					// Go away
					D3DXVECTOR3 new_v=ant_hill->get_pos()-_pos;
					_goto_pos.x=_pos.x-new_v.x*1.5f;
					_goto_pos.y=_pos.y-new_v.y*1.5f;
					_state=ANT_WANDER;
				}
			}
		}

		// Turn to the correct direction
		float angle=get_rotation_angle(norm_vec);
		float error1=angle-get_angle();
		float error2=angle-(get_angle()+360);
		float error;
		if (error2>error1) error=error1;
		else error=error2;
		angle=get_angle()+0.5f*error;
		set_angle(angle);
	}
	if (scr)
	{
		if (_state==ANT_GOTO)
		{
			PhenoromeMap *pm=scr->get_phenorome_map();
			pm->add_phenorome(_pos.x,_pos.y,PM_NORMAL_AMMOUNT);
		}
		else if (_state==ANT_FOLLOW)
		{
			PhenoromeMap *pm=scr->get_phenorome_map();
			pm->add_phenorome(_pos.x,_pos.y,PM_FOLLOW_AMMOUNT);
		}
		if ((_state!=ANT_RETURN) && (_state!=ANT_FLEE))
		{
			float d;
			Leaf* leaf=scr->find_nearest_leaf(_pos.x,_pos.y,&d);
			if (d<Sqr(50.0f))
			{
				// Drop phenorome bomb
				PhenoromeMap *pm=scr->get_phenorome_map();
				pm->add_phenorome_bomb(leaf->get_x_pos(),leaf->get_y_pos(),
									   _pos.x,_pos.y,
									   PM_NORMAL_AMMOUNT);
				// Catch leaf!
				leaf->eat();
				SOUND->play(SOUND->find_by_name("Chomp"));
				// Change animation (eventually)
				// Do particle system (eventually)
				ParticleSystem *part=new ParticleSystem;
				part->set_pos(_pos.x,_pos.y,_pos.z-0.1f);
				part->set_image(TEXTURES->find_by_name("ParticleLeaf"));
				part->set_life(250.0f);
				part->set_rate(100.0f,100.0f);
				part->set_particle_life(750.0f,250.0f);
				part->set_particle_speed(0.0001f,0.00005f);
				part->set_particle_size(0.01f,0.03f);
				part->set_particle_color_start(0.0f,0.7f,0.0f,1.0f);
				part->set_particle_color_end(0.0f,0.7f,0.0f,0.0f);
				part->set_blend_mode(BLEND_ALPHA);
				SCREEN->add_drawable(part);
				// Return to ant hill
				AntHill* spr=scr->find_nearest_anthill(_pos.x,_pos.y);
				if (spr)
				{
					go_to(spr->get_x_pos(),spr->get_y_pos(),2.0f);
					_state=ANT_RETURN;
				}
				else wander();
			}
		}
		if (_state!=ANT_FLEE)
		{
			float d;
			Grasshoper* gh=scr->find_nearest_grasshoper(_pos.x,_pos.y,&d);
			if (gh)
			{
				if (d<Sqr(200.0f))
				{
					queue_goto();
					D3DXVECTOR3 v=gh->get_pos()-_pos; v.z=0.0f;
					D3DXVECTOR3 v_norm;
					D3DXVec3Normalize(&v_norm,&v);
					v=_pos-v_norm*100.0f;
					_goto_pos=D3DXVECTOR2(v.x,v.y);
					_state=ANT_FLEE;
				}
			}
		}
		else
		{
			float d;
			Grasshoper* gh=scr->find_nearest_grasshoper(_pos.x,_pos.y,&d);
			if (gh)
			{
				if (d<Sqr(200.0f))
				{
					D3DXVECTOR3 v=gh->get_pos()-_pos; v.z=0.0f;
					D3DXVECTOR3 v_norm;
					D3DXVec3Normalize(&v_norm,&v);
					v=_pos-v_norm*100.0f;
					_goto_pos=D3DXVECTOR2(v.x,v.y);
					_state=ANT_FLEE;
				}
				else
				{
					if (_goto_queue.empty())
					{
						wander();
					}
					else
					{
						_goto_pos=_goto_queue.front();
						_goto_queue.pop_front();
					}
				}
			}
			else
			{
				if (_goto_queue.empty())
				{
					wander();
				}
				else
				{
					_goto_pos=_goto_queue.front();
					_goto_queue.pop_front();
				}
			}
		}
		float d;
		AntEater* ae=scr->find_nearest_ant_eater(_pos.x,_pos.y,&d);
		if (ae)
		{
			if (d<Sqr(50.0f))
			{
				// This ant is food!
				SOUND->play(SOUND->find_by_name("Chomp2"));
				scr->remove_drawable(this);
				scr->remove_ant(this);
				delete this;
				return;
			}
		}
	}
}


void Ant::go_to(float x,float y,float speed)
{
	_state=ANT_GOTO;
	_goto_pos.x=x;
	_goto_pos.y=y;
	_speed=speed;
}

void Ant::stop()
{
	_state=ANT_STOP;
}

void Ant::wander()
{
	_state=ANT_WANDER;
	_speed=1.0f;
	GameScreen* scr=dynamic_cast<GameScreen*>(SCREEN);
	if (scr)
	{
		AntHill* ant_hill=scr->find_nearest_anthill(_pos.x,_pos.y);
		D3DXVECTOR2	v,v_norm;
		v.x=ant_hill->get_x_pos()-_pos.x;
		v.y=ant_hill->get_y_pos()-_pos.y;
		D3DXVec2Normalize(&v_norm,&v);
		float	base=get_rotation_angle(v_norm);
		float	angle=deg_to_rad(get_random_number(base-25.0f,base+25.0f));
		float	c=cosf(angle);
		float	s=sinf(angle);
		float	d=get_random_number(100.0f,250.0f);
		_goto_pos.x=ant_hill->get_x_pos()+d*c;
		_goto_pos.y=ant_hill->get_y_pos()+d*s;
		if (_goto_pos.x<30.0f) _goto_pos.x=30.0f;
		else if (_goto_pos.x>990.0f) _goto_pos.x=990.0f;
		if (_goto_pos.y<30.0f) _goto_pos.y=30.0f;
		else if (_goto_pos.y>730.0f) _goto_pos.y=730.0f;
	}
}

void Ant::queue_goto()
{
	_goto_queue.push_back(_goto_pos);
}
