// GameScreen.h: interface for the GameScreen class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_GAMESCREEN_H__F2726937_025C_4CB4_8970_4D6527F32064__INCLUDED_)
#define AFX_GAMESCREEN_H__F2726937_025C_4CB4_8970_4D6527F32064__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "../framework/Screen.h"
#include "../framework/Drawable.h"
#include "../framework/Sprite.h"

#include <list>

class Ant;
class AntHill;
class PhenoromeMap;
class Leaf;
class Grasshoper;
class AntEater;

class GameScreen : public Screen  
{
protected:
public:
	class PriorityOrderFunc 
	{
	public:
		bool operator()(Drawable* o1,Drawable* o2)
		{
			if ((!o1) && (o2)) return true;
			else if ((o1) && (!o2)) return false;
			else if ((!o1) && (!o2)) return false;
			return (o1->get_priority())>(o2->get_priority());
		}
	};
	typedef std::list<Ant*>			Ants;
	typedef std::list<AntHill*>		AntHills;
	typedef std::list<Leaf*>		Leafs;
	typedef std::list<Grasshoper*>	Grasshopers;
	typedef std::list<AntEater*>	AntEaters;
public:
	GameScreen();
	virtual ~GameScreen();

			bool	init(long level);
			bool	read_assets();
			bool	load_level();
			void	shutdown();

	virtual void	render();
	virtual void	anim();

	virtual	void	mouse_callback(unsigned long msg,unsigned long lparam,unsigned long uparam);

public:
			Ant*			get_selected_ant() { return _selected; }
			PhenoromeMap*	get_phenorome_map() { return _pm; }
			bool			is_ant_on(float x,float y,Ant* exclude);
			AntHill*		find_nearest_anthill(float x,float y,float* distance=NULL);
			Leaf*			find_nearest_leaf(float x,float y,float* distance=NULL);
			Grasshoper*		find_nearest_grasshoper(float x,float y,float* distance=NULL);
			AntEater*		find_nearest_ant_eater(float x,float y,float* distance=NULL);
			void			remove_ant(Ant* ant);
			void			remove_leaf(Leaf* leaf);
			void			remove_grasshoper(Grasshoper* grasshoper);
			void			remove_ant_eater(AntEater* ant_eater);
	inline	float			get_decay_time() { return _decay_time; }
protected:
			Ant*	get_ant(float x,float y,float radius);
			void	spawn_leaf();
			void	spawn_grasshoper();
			void	spawn_ant_eater();
protected:
	Drawable*		_camera_focus;
	float			_last_frame_time;
	unsigned long	_level;

	// Level def stuff
	unsigned long	_n_ants;
	AntHills		_ant_hills;
	float			_leaf_rate;
	float			_grasshoper_rate;
	float			_ant_eater_rate;
	float			_decay_time;
	float			_time_to_end;
	// Dynamic game data
	Ants			_ants;
	Ant*			_selected;
	Grasshopers		_grasshopers;
	AntEaters		_ant_eaters;
	Leafs			_leafs;
	PhenoromeMap*	_pm;
	float			_time_to_next_leaf;
	float			_time_to_next_grasshoper;
	float			_time_to_next_ant_eater;
};

#endif // !defined(AFX_GAMESCREEN_H__F2726937_025C_4CB4_8970_4D6527F32064__INCLUDED_)
