#include "../framework/TextureManager.h"
#include "../framework/Utils.h"
#include "../framework/BlitManager.h"

#include "Leaf.h"
#include "GameScreen.h"
#include "PhenoromeMap.h"

Leaf::Leaf(float x,float y,float z) : Sprite(x,y,z)
{
	set_angle(get_random_number(0,360));
	set_image(TEXTURES->find_by_name("Food01"));
	set_priority(-50.0f);
	set_hotspot(32.0f,32.0f);
	_feed=100.0f;
	_decay=0.0f;
	_color=D3DXVECTOR4(0.0f,1.0f,0.0f,1.0f);
	_last_decay=timeGetTime();

	_falling=true;
	_target_x=x;
	_target_y=y;
	_fall_angle=0;

	_pos.y=_pos.y-800.0f;
}

Leaf::~Leaf(void)
{
}

void Leaf::anim()
{
	float t=timeGetTime()-_last_decay;

	if (_falling)
	{
		_pos.x=_target_x+140.0f*sin(deg_to_rad(_fall_angle)); _fall_angle+=(720/5000.0f)*t;
		_pos.y+=(800.0f/5000.0f)*t;
		if (_pos.y>=_target_y)
		{
			_falling=false;
		}
	}
	else
	{
		GameScreen* scr=dynamic_cast<GameScreen*>(SCREEN);

		if (scr)
		{
			_decay+=t/scr->get_decay_time();
		}
		else
		{
			_decay+=t/5000.0f;
		}
		if (_decay<0.75f) _color.x=_decay*1.0f/0.75f;
		else 
		{
			_color.x=1.0f;
			if (_decay<=1.0f) _color.w=1.0f-4.0f*(_decay-0.75f);
		}
		if (_decay>1.0f)
		{
			GameScreen* scr=dynamic_cast<GameScreen*>(SCREEN);
			if (scr)
			{
				scr->remove_leaf(this);
			}
			SCREEN->remove_drawable(this);
			delete this;
			return;
		}
	}
	_last_decay=timeGetTime();
}

void Leaf::eat()
{
	_feed-=25.0f;
	if (_feed>0.0f)
	{
		if (_feed>75.0f) set_image(TEXTURES->find_by_name("Food01"));
		else if (_feed>50.0f) set_image(TEXTURES->find_by_name("Food02"));
		else if (_feed>25.0f) set_image(TEXTURES->find_by_name("Food03"));
		else set_image(TEXTURES->find_by_name("Food04"));
	}
	else
	{
		GameScreen* scr=dynamic_cast<GameScreen*>(SCREEN);
		if (scr)
		{
			scr->remove_leaf(this);
		}
		SCREEN->remove_drawable(this);
		delete this;
		return;
	}
}

void Leaf::draw()
{
	if (!_draw) return;

	BLIT->add(TEXTURES->find_by_name("LeafShadow"),
			  _pos.x-32.0f,_target_y-32.0f,_pos.z+0.05f,
			  64,64,1,1,1,0.5f*_color.w);

	Sprite::draw();
}
