// MenuScreen.cpp: implementation of the MenuScreen class.
//
//////////////////////////////////////////////////////////////////////

#include "MenuScreen.h"
#include "../framework/TextureManager.h"
#include "../framework/BlitManager.h"
#include "../framework/Gfx.h"
#include "../framework/Keyboard.h"
#include "../framework/Sound.h"
#include "../framework/MouseCursor.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

MenuScreen::MenuScreen()
{
	_fade_inc=0.001f;
	_color=D3DXVECTOR4(0.0f,0.0f,0.0f,1.0f);
}

MenuScreen::~MenuScreen()
{
	TEXTURES->clear_all();
	BLIT->clear_cache();
}

bool MenuScreen::init()
{
	if (TEXTURES->load("Background","images/mainmenu.jpg")==-1) return false;
	if (TEXTURES->load("Cursor","images/cursor.dds")==-1) return false;
	
	_time_since_last_frame=timeGetTime();

	create_font();
	add_drawable(new MouseCursor);

	if (SOUND->get_sound_count()==0)
	{
		load_assets("data/SoundAssets.txt");
		SOUND->play(SOUND->find_by_name("IntroMusic"));
	}

	_opt=-1;

	return true;
}

void MenuScreen::render()
{
	GFX->clear(D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER,0.0f,0.0f,0.0f,0.0f,1.0f,0);
	GFX->set_2d_render();
	GFX->begin_scene();
	GFX->set_camera(512.0f,384.0f,0.0f);

	BLIT->clear();
	BLIT->add(TEXTURES->find_by_name("Background"),0.0f,0.0f,0.5f,1024.0f,768.0f,_color.x,_color.y,_color.z,1.0f);

	D3DXVECTOR3 cm=GFX->get_color_modulation();
	D3DXVECTOR4	c[3];
	for (int i=0; i<3; i++) { c[i].x=0.0f*cm.x; c[i].y=0.0f*cm.y; c[i].z=0.0f*cm.z; c[i].w=1.0f; }

	if (_fade_inc>=0.0f)
	{
		long old_opt=_opt;
		_opt=-1;
		if ((CURSOR->get_x_pos()>300.0f) && (CURSOR->get_x_pos()<400.0f))
		{
			float y=CURSOR->get_y_pos();
			if ((y>450) && (y<490)) { _opt=0; }
			else if ((y>500) && (y<540)) { _opt=1; }
			else if ((y>550) && (y<590)) { _opt=2; }
		}
		if ((_opt>=0) && (_opt<=2) && (old_opt!=_opt))
		{
			SOUND->play(SOUND->find_by_name("MouseOver"));
		}
	}
	if (_opt!=-1) c[_opt].y=1.0f*cm.y*_color.y;

	for (size_t i=0; i<_drawables.size(); i++)
	{
		if (_drawables[i]) _drawables[i]->draw();
	}

	BLIT->render();
	BLIT->clear();

	write_text(300,450,c[0].x,c[0].y,c[0].z,c[0].w,"Start");
	write_text(300,500,c[1].x,c[1].y,c[1].z,c[1].w,"Help");
	write_text(300,550,c[2].x,c[2].y,c[2].z,c[2].w,"Exit");

	GFX->end_scene();
}

void MenuScreen::anim()
{
	float t=timeGetTime()-_time_since_last_frame;
	if (t>50.0f) t=50.0f;

	_color.x=_color.x+_fade_inc*t; if (_color.x<0.0f) { _color.x=0.0f; set_exit(true,_opt); } else if (_color.x>1.0f) { _color.x=1.0f; _fade_inc=0.0f; }
	_color.y=_color.y+_fade_inc*t; if (_color.y<0.0f) { _color.y=0.0f; set_exit(true,_opt); } else if (_color.y>1.0f) { _color.y=1.0f; _fade_inc=0.0f; }
	_color.z=_color.z+_fade_inc*t; if (_color.z<0.0f) { _color.z=0.0f; set_exit(true,_opt); } else if (_color.z>1.0f) { _color.z=1.0f; _fade_inc=0.0f; }

	for (size_t i=0; i<_drawables.size(); i++)
	{
		if (_drawables[i]) _drawables[i]->anim();
	}

	_time_since_last_frame=float(timeGetTime());
}

void MenuScreen::mouse_callback(unsigned long msg,unsigned long lparam,unsigned long uparam)
{
	switch (msg)
	{
		case WM_LBUTTONUP:
			{
				if ((_opt>=0) && (_opt<=2) && (_fade_inc>=0.0f))
				{
					SOUND->play(SOUND->find_by_name("MenuSelect"));
					_fade_inc=-0.001f;
				}
			}
			break;
	}
}