#include "PhenoromeMap.h"
#include "GameScreen.h"
#include "../Framework/Utils.h"
#include "../Framework/TextureManager.h"
#include "../Framework/BlitManager.h"
#include "../Framework/Gfx.h"

PhenoromeMap::PhenoromeMap(void)
{
	_map=(float*)malloc(sizeof(float)*PM_RESX*PM_RESY);
	memset(_map,0,sizeof(float)*PM_RESX*PM_RESY);
	_color.x=0.0f;
	_color.y=0.5f;
	_color.z=1.0f;
	_color.w=0.75f;
	_texture=NULL;
	build_texture();
	if (_texture) _texture_id=TEXTURES->add("Phenorome",_texture);
	set_priority(-500.0f);
}

PhenoromeMap::~PhenoromeMap(void)
{
	if (_map) free(_map); _map=NULL;
}

void PhenoromeMap::draw()
{
	if (!_draw) return;

	GFX->get_device()->SetRenderState(D3DRS_ALPHATESTENABLE,0);
	D3DXVECTOR3 color=GFX->get_color_modulation();
	color.x*=_color.x;
	color.y*=_color.y;
	color.z*=_color.z;
	BLIT->add(_texture_id,0.0f,0.0f,0.2f,1024,768,color.x,color.y,color.z,_color.w);
}

void PhenoromeMap::anim()
{
	// Fill the phenorome map
	D3DLOCKED_RECT	tmp;
	HRESULT hRet=_texture->LockRect(0,&tmp,NULL,D3DLOCK_DISCARD|D3DLOCK_NOSYSLOCK);
	if (hRet!=D3D_OK) return;

	unsigned char	c;
	unsigned char*	ptr;
	for (unsigned long y=0; y<PM_RESY; y++)
	{
		ptr=(unsigned char*)tmp.pBits;
		ptr+=tmp.Pitch*y;

		for (unsigned long x=0; x<PM_RESX; x++)
		{
			// Decay
			_map[y*PM_RESX+x]-=PM_DECAY; 
			if (_map[y*PM_RESX+x]<0.0f) _map[y*PM_RESX+x]=0.0f;

			c=unsigned char(_map[y*PM_RESX+x]*255.0f/PM_UPPERBOUND);
			*ptr=255; ptr++;
			*ptr=255; ptr++;
			*ptr=255; ptr++;
			*ptr=c; ptr++;
		}
	}

	_texture->UnlockRect(0);
}

void PhenoromeMap::build_texture()
{
	if (!_texture)
	{
		HRESULT hRet=GFX->get_device()->CreateTexture(PM_RESX,PM_RESY,1,
													  0,
													  D3DFMT_A8R8G8B8,
													  D3DPOOL_MANAGED,
													  &_texture,NULL);
		if (hRet!=D3D_OK)
		{
			ERROR_MSG("Can't create texture for phenorome map!");
			return;
		}
	}
}

void PhenoromeMap::add_phenorome(float x,float y,float ammount)
{
	D3DXVECTOR2 grid_pos=to_grid(x,y);
	if ((grid_pos.x<0) || (grid_pos.x>=PM_RESX)) return;
	if ((grid_pos.y<0) || (grid_pos.y>=PM_RESY)) return;
	
	float tmp=_map[long(grid_pos.y)*PM_RESX+long(grid_pos.x)];
	tmp+=ammount;
	if (tmp>PM_UPPERBOUND) tmp=PM_UPPERBOUND;
	else if (tmp<0.0f) tmp=0.0f;

	_map[long(grid_pos.y)*PM_RESX+long(grid_pos.x)]=tmp;
}

void PhenoromeMap::add_phenorome_bomb(float x,float y,
									  float tx,float ty,
									  float ammount)
{
	D3DXVECTOR2 grid_pos=to_grid(x,y);
	D3DXVECTOR2 grid_pos2=to_grid(tx,ty);
	long		radius_sqr=long(Sqr(grid_pos.x-grid_pos2.x)+Sqr(grid_pos.y-grid_pos2.y));
	long		radius=long(sqrtf(radius_sqr))*1.5;

	for (long xx=grid_pos.x-radius; xx<=grid_pos.x+radius; xx++)
	{
		for (long yy=grid_pos.y-radius; yy<=grid_pos.y+radius; yy++)
		{
			if ((xx>=0) && (xx<PM_RESX))
			{
				if ((yy>=0) && (yy<PM_RESY))
				{
					float grid_distance=sqrt(Sqr(xx-grid_pos.x)+Sqr(yy-grid_pos.y));
					if (grid_distance<radius)
					{
						float tmp=_map[long(yy)*PM_RESX+long(xx)];
						tmp+=ammount*(1.1f-(grid_distance/radius));
						if (tmp>PM_UPPERBOUND) tmp=PM_UPPERBOUND;
						else if (tmp<0.0f) tmp=0.0f;

						_map[long(yy)*PM_RESX+long(xx)]=tmp;
					}
				}
			}
		}
	}
}

D3DXVECTOR2 PhenoromeMap::to_grid(float x,float y)
{
	long grid_x=(x/1024.0f)*PM_RESX;
	long grid_y=(y/768.0f)*PM_RESY;

	return D3DXVECTOR2(grid_x,grid_y);
}

bool PhenoromeMap::get_scent(float x,float y,D3DXVECTOR2* v,Ant* ant)
{
	D3DXVECTOR2	gp=to_grid(x,y);
	D3DXVECTOR2 scale;
	scale.x=1024.0f/PM_RESX;
	scale.y=768.0f/PM_RESY;

	GameScreen* scr=dynamic_cast<GameScreen*>(SCREEN);

	float m=0.0f;
	for (int xx=-6; xx<=6; xx++)
	{
		for (int yy=-6; yy<=6; yy++)
		{
			if ((xx!=0) || (yy!=0))
			{
				float tmp=get_value(gp.x+xx,gp.y+yy);
				if (tmp>m)
				{
					float new_x=x+scale.x*xx;
					float new_y=y+scale.y*yy;

					if (scr)
					{
						if (!scr->is_ant_on(new_x,new_y,ant))
						{
							m=tmp;
							v->x=new_x;
							v->y=new_y;
						}
					}
				}
			}
		}
	}
	if (m!=0.0f) 
	{
//		v->x=v->x+scale.x*0.5f;
//		v->y=v->y+scale.y*0.5f;
		return true;
	}

	return false;
}

float PhenoromeMap::get_value(float x,float y)
{
	if ((x<0) || (x>=PM_RESX)) return 0.0f;
	if ((y<0) || (y>=PM_RESY)) return 0.0f;

	return _map[long(y)*PM_RESX+long(x)];
}
